@@ -1,12 +1,9 @@
 import os
 import warnings
-from functools import partial
 from multiprocessing.pool import ThreadPool
 from typing import Optional, Dict
 
 import onnxruntime as ort
-from jina import Executor, requests, DocumentArray
-
 from clip_server.executors.helper import (
     split_img_txt_da,
     preproc_image,
@@ -15,6 +12,7 @@ from clip_server.executors.helper import (
 )
 from clip_server.model import clip
 from clip_server.model.clip_onnx import CLIPOnnxModel
+from jina import Executor, requests, DocumentArray
 
 
 class CLIPEncoder(Executor):
@@ -23,16 +21,18 @@ class CLIPEncoder(Executor):
         name: str = 'ViT-B/32',
         device: Optional[str] = None,
         num_worker_preprocess: int = 4,
-        minibatch_size: int = 16,
+        minibatch_size: int = 32,
+        traversal_paths: str = '@r',
         **kwargs,
     ):
         super().__init__(**kwargs)
 
+        self._minibatch_size = minibatch_size
+        self._traversal_paths = traversal_paths
+
         self._preprocess_tensor = clip._transform_ndarray(clip.MODEL_SIZE[name])
         self._pool = ThreadPool(processes=num_worker_preprocess)
 
-        self._minibatch_size = minibatch_size
-
         self._model = CLIPOnnxModel(name)
 
         import torch
@@ -48,8 +48,6 @@ class CLIPEncoder(Executor):
         # prefer CUDA Execution Provider over CPU Execution Provider
         if self._device.startswith('cuda'):
             providers.insert(0, 'CUDAExecutionProvider')
-            # TODO: support tensorrt
-            # providers.insert(0, 'TensorrtExecutionProvider')
 
         sess_options = ort.SessionOptions()
 
@@ -63,7 +61,7 @@ class CLIPEncoder(Executor):
             and hasattr(self.runtime_args, 'replicas')
         ):
             replicas = getattr(self.runtime_args, 'replicas', 1)
-            num_threads = max(1, torch.get_num_threads() // replicas)
+            num_threads = max(1, torch.get_num_threads() * 2 // replicas)
             if num_threads < 2:
                 warnings.warn(
                     f'Too many replicas ({replicas}) vs too few threads {num_threads} may result in '
@@ -79,6 +77,22 @@ class CLIPEncoder(Executor):
 
         self._model.start_sessions(sess_options=sess_options, providers=providers)
 
+    def _preproc_images(self, docs: 'DocumentArray'):
+        with self.monitor(
+            name='preprocess_images_seconds',
+            documentation='images preprocess time in seconds',
+        ):
+            return preproc_image(
+                docs, preprocess_fn=self._preprocess_tensor, return_np=True
+            )
+
+    def _preproc_texts(self, docs: 'DocumentArray'):
+        with self.monitor(
+            name='preprocess_texts_seconds',
+            documentation='texts preprocess time in seconds',
+        ):
+            return preproc_text(docs, return_np=True)
+
     @requests(on='/rank')
     async def rank(self, docs: 'DocumentArray', parameters: Dict, **kwargs):
         await self.encode(docs['@r,m'])
@@ -86,41 +100,40 @@ class CLIPEncoder(Executor):
         set_rank(docs)
 
     @requests
-    async def encode(self, docs: 'DocumentArray', **kwargs):
+    async def encode(self, docs: 'DocumentArray', parameters: Dict = {}, **kwargs):
+
+        traversal_paths = parameters.get('traversal_paths', self._traversal_paths)
+        minibatch_size = parameters.get('minibatch_size', self._minibatch_size)
+
         _img_da = DocumentArray()
         _txt_da = DocumentArray()
-        for d in docs:
+        for d in docs[traversal_paths]:
             split_img_txt_da(d, _img_da, _txt_da)
 
         # for image
         if _img_da:
-            for minibatch, _contents in _img_da.map_batch(
-                partial(
-                    preproc_image, preprocess_fn=self._preprocess_tensor, return_np=True
-                ),
-                batch_size=self._minibatch_size,
+            for minibatch, batch_data in _img_da.map_batch(
+                self._preproc_images,
+                batch_size=minibatch_size,
                 pool=self._pool,
             ):
-                minibatch.embeddings = self._model.encode_image(minibatch.tensors)
-                # recover original content
-                if _contents:
-                    for _d, _ct in zip(minibatch, _contents):
-                        _d.content = _ct
+                with self.monitor(
+                    name='encode_images_seconds',
+                    documentation='images encode time in seconds',
+                ):
+                    minibatch.embeddings = self._model.encode_image(batch_data)
 
         # for text
         if _txt_da:
-            for minibatch, _contents in _txt_da.map_batch(
-                partial(preproc_text, return_np=True),
-                batch_size=self._minibatch_size,
+            for minibatch, batch_data in _txt_da.map_batch(
+                self._preproc_texts,
+                batch_size=minibatch_size,
                 pool=self._pool,
             ):
-                minibatch.embeddings = self._model.encode_text(minibatch.tensors)
-                # recover original content
-                if _contents:
-                    for _d, _ct in zip(minibatch, _contents):
-                        _d.content = _ct
-
-        # drop tensors
-        docs.tensors = None
+                with self.monitor(
+                    name='encode_texts_seconds',
+                    documentation='texts encode time in seconds',
+                ):
+                    minibatch.embeddings = self._model.encode_text(batch_data)
 
         return docs
